import os

from hilster_theme import _version as version
import requests
import datetime


def get_versions(product, base_url="https://docs.hilster.io/"):
    """
    Return a list with all versions that have docs online for a given job.

    Args:
        product (str): the name of the product
        base_url (str): the docs base url
    """
    url = f'{base_url}{product}/versions.php'
    session = requests.Session()
    ret = session.get(url)

    assert ret.status_code == 200, "Status is not 200"

    data = ret.json()

    versions = []

    if 'latest' in data:
        del data[data.index('latest')]

    if 'discontinued' in data:
        del data[data.index('discontinued')]

    versions += list(reversed(data))

    return versions


# cache for versions
versions = None


def get_version_tuples(product, base_url="https://docs.hilster.io/"):
    """
    Return a sorted tuple containing all versions and urls for all versions.
    'latest' marks the latest version.

    Args:
        product (str): the name of the product
        base_url (str): the docs base url
    """

    global versions

    if not product:
        # do not return any versions if the product is not set
        # version selection will not appear in generated docs
        return versions

    # cache
    if versions:
        return versions

    try:
        all_versions = get_versions(product, base_url)
    except Exception:
        print("versions for", product, "could not be fetched!")
        all_versions = []

    versions = [('latest', f"{base_url}{product}/latest")]

    for v in all_versions:
        # remove product from version
        slug = v

        if slug.startswith(product):
             slug = slug[len(product) + 1:]

        versions.append((slug, f"{base_url}{product}/{v}"))

    print(f"versions for '{product}':", versions)

    return versions


def get_path():
    """
    Shortcut for users whose theme is next to their conf.py.
    """
    # Theme directory is defined as our parent directory
    return os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


def update_context(app, pagename, templatename, context, doctree):
    context['hilster_theme_version'] = version.__version__
    context['versions'] = get_version_tuples(context['theme_product'])
    context['current_version'] = context['version']
    context['year'] = datetime.datetime.now().year


def setup(app):
    app.setup_extension("sphinx_copybutton")
    app.connect('html-page-context', update_context)
    return {'version': version.__version__,
            'parallel_read_safe': True}
